from typing import List

import os
import pathlib

from bluetarget.errors import FileNotFound, ModelClassNotFound


def validate_files(files: List[str]) -> None:
    for file in files:
        if os.path.exists(file) == False:
            raise FileNotFound(file)


def validate_model_class(model_class: str, files: List[str]) -> str:
    files = list(filter(lambda f: pathlib.Path(f).suffix == ".py", files))

    for file in files:
        text_file = open(file, "r")
        raw_data = text_file.read()
        text_file.close()

        if raw_data.find(model_class) != -1:
            return file

    raise ModelClassNotFound(model_class)





from typing import Dict, List, Tuple
from bluetarget.model_version import ModelVersion

from bluetarget.validation import validate_files, validate_model_class

from bluetarget.utils import clean_tmp, prepare_package
from bluetarget.model import Model


class BlueTarget:

    api_key: str
    model_id: str
    model_version_id: str

    model: Model

    def __init__(self, api_key: str, model_id: str = None, model_version_id: str = None) -> None:
        self.api_key = api_key
        self.model_id = model_id
        self.model_version_id = model_version_id

        self.model = Model(api_key)

    def set_model_id(self, model_id: str) -> None:
        self.model_id = model_id
        self.model.set_model_id(model_id)

    def set_model_version_id(self, model_version_id: str) -> None:
        self.model_version_id = model_version_id

    def deploy(self, model_class: str, model_files: List[str], requirements_file: str, model_name: str = None, metadata: Dict = None, environment: Dict = None, algorithm: str = None, implementation: str = None) -> Tuple[Model, ModelVersion]:

        if self.model_id == None:
            self.model.create(name=model_name)
        else:
            self.model.get(self.model_id)

        if self.model_version_id == None:
            model_version = self.model.create_version(
                model_class=model_class,
                model_files=model_files,
                metadata=metadata,
                requirements_file=requirements_file,
                algorithm=algorithm,
                environment=environment,
                implementation=implementation
            )
        else:
            model_version = self.model.get_version(self.model_version_id)

        files = [*model_files, requirements_file]

        validate_files(files)
        validate_model_class(model_class, model_files)

        package = prepare_package(files)

        model_version.upload_package(package_url=package)

        model_version.deploy()

        clean_tmp()

        return self.model, model_version

    def predict(self, inputs: List):
        return self.model.predict(inputs=inputs)

    def health(self):
        return self.model.health()





from typing import Dict, List

from bluetarget.api_endpoint import APIEndpoint
from bluetarget.errors import AuthorizationError, EntityNotFound, ServerValidationException
from bluetarget.model_version import ModelVersion


class Model:
    api_key: str
    model_id: str
    endpoint: APIEndpoint
    data: Dict

    def __init__(self, api_key: str, model_id: str = None) -> None:
        self.api_key = api_key
        self.endpoint = APIEndpoint(api_key)
        self.model_id = model_id

    def set_model_id(self, id):
        self.model_id = id

    def get(self, model_id: str):
        response, status = self.endpoint.get(f"models/{model_id}")

        if status == 403:
            raise AuthorizationError()

        if status == 404:
            raise EntityNotFound("Model", model_id)

        if status != 200:
            raise ServerValidationException(
                response['code'], response['description'])

        self.data = response
        self.set_model_id(response['id'])

        return response

    def create(self, name: str, description: str = None, serverId: str = None, replicas: int = None):

        body = {
            "name": name
        }

        if description != None:
            body["description"] = description

        if serverId != None:
            body["serverId"] = serverId

        if replicas != None:
            body["replicas"] = replicas

        response, status = self.endpoint.post("models/", body=body)

        if status == 403:
            raise AuthorizationError()

        if status != 200:
            raise ServerValidationException(
                response['code'], response['description'])

        self.data = response
        self.set_model_id(response['id'])

        return response

    def create_version(self, model_class: str, model_files: List[str], requirements_file: str, metadata: Dict = None, environment: Dict = None, algorithm: str = None, implementation: str = None) -> ModelVersion:
        model_version = ModelVersion(
            api_key=self.api_key, model_id=self.data["id"])

        model_version.create(
            model_class=model_class,
            model_files=model_files,
            metadata=metadata,
            requirements_file=requirements_file,
            algorithm=algorithm,
            environment=environment,
            implementation=implementation
        )

        return model_version

    def get_version(self, model_version_id: str):
        model_version = ModelVersion(
            api_key=self.api_key, model_id=self.model_id)

        model_version.get(model_version_id=model_version_id)

        return model_version

    def health(self):
        return self.endpoint.get(
            f"models/{self.model_id}/health")

    def predict(self, inputs: List):
        body = {
            "inputs": inputs
        }
        return self.endpoint.post(
            f"models/{self.model_id}/predict", body)




from typing import Dict

import requests
import json

ENDPOINT = "https://api.deploy.bluetarget.ai"


class APIEndpoint:
    def __init__(self, api_key: str) -> None:
        self.api_key = api_key

    def get(self, path: str, query: Dict = None):
        response = requests.get(f'{ENDPOINT}/{path}',
                                params=query, headers=self.__headers())

        return response.json(), response.status_code

    def post(self, path: str, body: Dict = None):

        if body != None:
            body = json.dumps(body)

        response = requests.post(f'{ENDPOINT}/{path}',
                                 data=body, headers=self.__headers())

        return response.json(), response.status_code

    def __headers(self):
        return {
            "Authorization": f"api-key {self.api_key}",
            "Content-Type": "application/json"
        }



class FileNotFound(Exception):

    file: str

    def __init__(self, file: str):
        self.file = file

    def __str__(self):
        return f'File {self.file} not found'


class ModelClassNotFound(Exception):

    class_name: str

    def __init__(self, class_name: str):
        self.class_name = class_name

    def __str__(self):
        return f'Class {self.class_name} not found'


class EntityNotFound(Exception):

    entity: str
    id: str

    def __init__(self, entity: str, id: str):
        self.entity = entity
        self.id = id

    def __str__(self):
        return f'{self.entity} {self.id} not found'


class AuthorizationError(Exception):

    def __str__(self):
        return f'Api key not found'


class ServerValidationException(Exception):
    code: str
    description: str

    def __init__(self, code: str, description: str):
        self.code = code
        self.description = description

    def __str__(self):
        return f'${self.code}'



from typing import List

import os
import shutil
import zipfile
from pathlib import Path

from bluetarget.constants import CODE_DIR


def clean_tmp():
    shutil.rmtree(CODE_DIR)


def prepare_package(files: List[str]) -> str:

    if Path.exists(CODE_DIR) == False:
        os.mkdir(CODE_DIR)

    package = f'{CODE_DIR}/package.zip'

    with zipfile.ZipFile(package, mode="w") as archive:
        for file in files:
            archive.write(file)

    return package



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['bluetarget']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'bluetarget',
    'version': '0.1.5',
    'description': 'Deploy, monitor and explain your machine learning models',
    'long_description': '# BlueTarget\n\n**Deploy, monitor and explain machine learning models.**\n\n![Logo](https://deploy.bluetarget.ai/statics/bt.jpg)\n\nBlueTarget is a MLops platform which allows ML engineer and Data Science deploy, monitor and explain their machine learning models. We\'re able to deploy your model using different kind of strategies like A/B testing, Canary or Rolling update.\n\nGet started with the [BlueTarget](https://docs.deploy.bluetarget.ai).\n\n## What can I do with BlueTarget?\n\nIf you\'ve ever tried to get a model out of a Jupyter notebook, BlueTargert is for you.\n\nBlueTarget allow your to deploy your ML model taking away the whole complexity of the cloud. However If you prefer to have the control of the infrastructure, BlueTarget can work with your preferred cloud:\n\nHere are some of the things BlueTarget does:\n\n- Turns your Python model into a microservice with a production-ready API endpoint, no need for Flask or Django.\n- Track your model\'s version and metadata\n- Understad the drift of your model\n- Track your inference\n- Deployment strategies like A/B testing, canary and rolling update\n\n## Installation\n\nBlueTarget requires Python >=3.7\n\nTo install from [PyPi](https://pypi.org/project/bluetarget/), run:\n\n```\npip install bluetarget\n```\n\nBlueTarget is actively developed, and we recommend using the latest version. To update your BlueTarget installation, run:\n\n```\npip install --upgrade bluetarget\n```\n\n## How to use BlueTarget\n\n### Quickstart: making a BlueTarget\n\n#### train.py\n\n```python\n!pip install --upgrade scikit-learn bluetarget pickle-mixin\n\nfrom sklearn import svm\nfrom sklearn import datasets\n\nimport pickle\n\n# Load training data set\niris = datasets.load_iris()\nX, y = iris.data, iris.target\n\n# Train the model\nclf = svm.SVC(gamma=\'scale\')\nclf.fit(X, y)\n\npickle.dump(clf, open(\'model.pkl\', \'wb\'))\n\n```\n\n#### service.py\n\n```python\nimport os\nfrom typing import Dict, List\n\nclass Model:\n    def __init__(self) -> None:\n        self._model = None\n\n    def load(self):\n        import pickle\n\n        with open(f"{os.path.dirname(__file__)}/model.pkl", \'rb\') as pickle_file:\n            self._model = pickle.load(pickle_file)\n\n    def predict(self, request: Dict) -> Dict[str, List]:\n        response = {}\n        inputs = request["inputs"]\n        result = self._model.predict(inputs).tolist()\n        response["predictions"] = result\n\n        return response\n```\n\n#### requirements.txt\n\n```\nscikit-learn==1.0.2\npickle-mixin==1.0.2\n```\n\n#### deploy.py\n\n```python\nfrom bluetarget import BlueTarget\n\nbt = BlueTarget(api_key="YOUR_API_KEY")\n\nbt.deploy(\n    model_name="YourFirstModel",\n    model_class="Model",\n    model_files=["model.py", "model.pkl"],\n    requirements_file="requirements.txt"\n)\n\ninputs = [\n    [6.9, 3.1, 5.1, 2.3],\n    [5.8, 2.7, 5.1, 1.9],\n    [6.8, 3.2, 5.9, 2.3],\n    [6.7, 3.3, 5.7, 2.5],\n    [6.7, 3.,  5.2, 2.3],\n    [6.3, 2.5, 5.,  1.9],\n    [6.5, 3.,  5.2, 2.],\n    [6.2, 3.4, 5.4, 2.3],\n    [5.9, 3.,  5.1, 1.8]\n]\n\nbt.predict(inputs)\n\n# {\n#     "predictions": [\n#         2,\n#         1,\n#         2,\n#         3,\n#         0,\n#         2,\n#         3,\n#         2,\n#         1\n#     ]\n# }\n\n```\n',
    'author': 'Sergio Kaz',
    'author_email': 'sergio@bluetarget.ai',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)





import os
from typing import Dict, List

import requests

from bluetarget.api_endpoint import APIEndpoint
from bluetarget.errors import AuthorizationError, EntityNotFound, FileNotFound, ServerValidationException


class ModelVersion:
    api_key: str
    model_id: str
    model_version_id: str
    endpoint: APIEndpoint
    data: Dict

    def __init__(self, api_key: str, model_id: str) -> None:
        self.api_key = api_key
        self.model_id = model_id
        self.endpoint = APIEndpoint(api_key)

    def get(self, model_version_id: str):

        response, status = self.endpoint.get(
            f"models/{self.model_id}/versions/{model_version_id}")

        if status == 403:
            raise AuthorizationError()

        if status == 404:
            raise EntityNotFound("Model version", model_version_id)

        if status != 200:
            raise ServerValidationException(
                response['code'], response['description'])

        self.data = response
        self.set_model_version_id(response)

        return response

    def set_model_version_id(self, data):
        self.model_version_id = data['id']

    def create(self, model_class: str, model_files: List[str], requirements_file: str, metadata: Dict = None, environment: Dict = None, algorithm: str = None, implementation: str = None):
        body = {
            "modelClass": model_class,
            "files": model_files,
            "requirementsFile": requirements_file
        }

        if metadata != None:
            body["metadata"] = metadata

        if environment != None:
            body["environment"] = environment

        if algorithm != None:
            body["algorithm"] = algorithm

        if implementation != None:
            body["implementation"] = implementation

        response, status = self.endpoint.post(
            f"models/{self.model_id}/version", body=body)

        if status == 403:
            raise AuthorizationError()

        if status == 404:
            raise EntityNotFound("Model", self.model_id)

        if status != 200:
            raise ServerValidationException(
                response['code'], response['description'])

        self.data = response
        self.set_model_version_id(response)

        return response

    def upload_package(self, package_url: str):

        if os.path.exists(package_url) == False:
            raise FileNotFound(package_url)

        presigned_url, status = self.endpoint.post(
            f"models/{self.model_id}/versions/{self.model_version_id}/upload")

        url = presigned_url["uploadUrl"]
        fields = presigned_url["formData"]

        with open(package_url, 'rb') as f:
            files = {'file': (package_url, f)}
            requests.post(url, data=fields, files=files)

    def deploy(self):
        response, status = self.endpoint.post(
            f"models/{self.model_id}/versions/{self.model_version_id}/deploy")

        if status != 200:
            raise ServerValidationException(
                response['code'], response['description'])
